export P=qtwebkit-qt5
export V=5.212
export B=1
export MAINTAINER=JuergenFischer

savelog() {
	local f=$1
	rm -f $f.9
	for i in $(seq 8 -1 0); do [ -f "$f.$i" ] && mv $f.$i $f.$(( i + 1 )); done
	[ -f $f ] && mv $f $f.0
}

savelog package.log
exec > >(tee package.log) 2>&1
date

set -e

if ! [ -d ../qtwebkit ]; then
	cd ..
	git clone --branch $V https://code.qt.io/qt/qtwebkit.git
	cd qtwebkit
	git apply ../osgeo4w/qtwebkit.diff
	cd ../osgeo4w
fi

[ -f ninja-win.zip ] || wget https://github.com/ninja-build/ninja/releases/download/v1.8.2/ninja-win.zip
if ! [ -x ./ninja.exe ]; then
	unzip ninja-win.zip ninja.exe
	chmod a+rx ninja.exe
fi

if ! [ -d "$OSGEO4W_REP" ]; then
        echo no repository
        exit 1
fi

fetchenv() {
        local IFS
        IFS=
        batch=$1
        shift

        if ! [ -f "$batch" ]; then
                echo "$batch not found"
                return 1
        fi

        srcenv=$(/bin/mktemp /tmp/srcenv.XXXXXXXXXX)
        dstenv=$(/bin/mktemp /tmp/dstenv.XXXXXXXXXX)
        diffenv=$(/bin/mktemp /tmp/diffenv.XXXXXXXXXX)
        envdiff=$(/bin/mktemp /tmp/envdiff.XXXXXXXXXX)
        cmd /c set >$srcenv
        cmd /c call $(/bin/cygpath -w $batch) "$@" \>nul 2\>nul \& set >$dstenv
        /usr/bin/cat <<EOF >$envdiff
/^+++/d;
/^[^+]/d;
s/^+//;
# s/\\\\/\\\\\\\\/g;
/^PATH=/ {
        s/\\\\/\\//g;
        s#\\([=;]\\)\\([a-zA-Z]\\):#\\1/cygdrive/\\2#g;
        s/;/:/g;
}
s/;/\\\\;/g;
s/^/export /;
s/=/='/;
s/\\\\r//g;
s/\r$/'/;
EOF
        /usr/bin/diff -u $srcenv $dstenv | /usr/bin/sed -f $envdiff >$diffenv
        . $diffenv
        PATH=$PATH:/cygdrive/c/windows32/system32:/cygdrive/c/windows:/bin
        rm -f $srcenv $dstenv $diffenv $envdiff
}

for ARCH in x86_64 x86; do
	export ARCH

	(
		case "$ARCH" in
		x86)
        		export OSGEO4W_ROOT=/c/OSGeo4W
        		export VCARCH=x86
			export ZLIB=zlib.lib
			;;

		x86_64)
			export OSGEO4W_ROOT=/c/OSGeo4W64
	        	export VCARCH=amd64
			export ZLIB=zdll.lib
			;;

		*)
        		echo invalid arch
        		exit 1
			;;

		esac

		export R=$OSGEO4W_REP/$ARCH/release/qt5/$P
		mkdir -p $R

		for i in libs pdb debug devel; do
			mkdir -p $R/$P-$i
			cp ../qtwebkit/LICENSE.LGPLv21 $R/$P-$i/$P-$i-$V-$B.txt
		done

		cat <<EOF >$R/$P-libs/setup.hint
sdesc: "WebKit for Qt5"
ldesc: "WebKit for Qt5"
maintainer: JuergenFischer
category: Libs
requires: qt5-libs icu-libs
EOF

		cat <<EOF >$R/$P-pdb/setup.hint
sdesc: "WebKit for Qt5 (pdb)"
ldesc: "WebKit for Qt5 (pdb)"
maintainer: JuergenFischer
category: Libs
requires: $P-libs
external-source: $P-libs
EOF

		cat <<EOF >$R/$P-devel/setup.hint
sdesc: "WebKit for Qt5 (development)"
ldesc: "WebKit for Qt5 (development)"
maintainer: JuergenFischer
category: Libs
requires: $P-libs
external-source: $P-libs
EOF

		fetchenv $OSGEO4W_ROOT/bin/o4w_env.bat
		fetchenv $OSGEO4W_ROOT/bin/qt5_env.bat

		fetchenv "/c/Program Files (x86)/Microsoft Visual Studio 14.0/VC/vcvarsall.bat" $VCARCH
		export PATH="$PATH:/c/Program Files/CMake/bin:$PWD:$(cygpath -m $PYTHONHOME)"

		export INSTDIR=$PWD/install-$ARCH BUILDDIR=$PWD/build-$ARCH
		[ -f noclean ] || rm -rf $BUILDDIR
		mkdir -p $INSTDIR $BUILDDIR

		cd $BUILDDIR

		export LIB="$(cygpath -w $OSGEO4W_ROOT/lib);$LIB"

		if ! [ -f CMakeCache.txt ]; then
			cmake -G Ninja \
				-Wno-dev \
				-DPORT=Qt \
				-DCMAKE_BUILD_TYPE=RelWithDebInfo \
				-DCMAKE_INSTALL_PREFIX=$(cygpath -m $INSTDIR) \
				-DPYTHON_EXECUTABLE=$(cygpath -m $OSGEO4W_ROOT/bin/python.exe) \
				-DSQLITE_LIBRARIES=$(cygpath -m $OSGEO4W_ROOT/lib/sqlite3_i.lib) \
				-DJPEG_LIBRARY="$(cygpath -m $OSGEO4W_ROOT/lib/jpeg_i.lib)" \
                        	-DPNG_PNG_INCLUDE_DIR="$(cygpath -m $OSGEO4W_ROOT/libs/vc14/include)" \
                        	-DPNG_LIBRARY="$(cygpath -m $OSGEO4W_ROOT/libs/vc14/lib/libpng16.lib)" \
				-DZLIB_LIBRARY="$(cygpath -m $OSGEO4W_ROOT/lib/$ZLIB)" \
				-DENABLE_XSLT=OFF \
				../../qtwebkit
		fi

		ninja
		ninja install

		cd ..

		/usr/bin/tar -C .. -cjvf $R/$P-libs/$P-libs-$V-$B-src.tar.bz2 \
			osgeo4w/package.sh \
			osgeo4w/qtwebkit.diff

		cd $INSTDIR

		/usr/bin/tar -cjvf $R/$P-devel/$P-devel-$V-$B.tar.bz2 \
			--xform s,^,apps/Qt5/, \
			--xform s,^apps/Qt5/lib/qml,apps/Qt5/qml, \
			--exclude "*.dll" \
			--exclude lib/qml/QtWebKit/plugins.qmltypes \
			include \
			lib \
			mkspecs

		/usr/bin/tar -cjvf $R/$P-libs/$P-libs-$V-$B.tar.bz2 \
			--xform s,^,apps/Qt5/, \
			--xform s,^apps/Qt5/lib/qml,apps/Qt5/qml, \
			bin/*.{dll,exe} \
			lib/qml/QtWebKit/experimental/qmlwebkitexperimentalplugin.dll \
			lib/qml/QtWebKit/plugins.qmltypes \
			lib/qml/QtWebKit/qmlwebkitplugin.dll

		/usr/bin/tar -cjvf $R/$P-pdb/$P-pdb-$V-$B.tar.bz2 \
			--xform s,^,apps/Qt5/, \
			bin/*.pdb
	)
done

date
